]> git.sesse.net Git - stockfish/blobdiff - src/tt.h
Fixed some warnings when using -Weffc++ gcc option
[stockfish] / src / tt.h
index 76ecad81e101254dea4832984f2022dd9e2055da..9babce8cdc9f10e14500c1f9c4e5cdc6a071d033 100644 (file)
--- a/src/tt.h
+++ b/src/tt.h
@@ -1,7 +1,7 @@
 /*
   Stockfish, a UCI chess playing engine derived from Glaurung 2.1
   Copyright (C) 2004-2008 Tord Romstad (Glaurung author)
-  Copyright (C) 2008 Marco Costalba
+  Copyright (C) 2008-2010 Marco Costalba, Joona Kiiski, Tord Romstad
 
   Stockfish is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 ////
 
 #include "depth.h"
-#include "position.h"
+#include "move.h"
 #include "value.h"
 
 
 ///
 /// A TTEntry needs 128 bits to be stored
 ///
/// bit    0-63: key
/// bit   64-95: data
/// bit  96-111: value
/// bit 112-127: depth
 ///
 /// the 32 bits of the data field are so defined
 ///
 /// bit  0-16: move
/// bit 17-19: not used
/// bit 20-22: value type
 /// bit 23-31: generation
 
 class TTEntry {
 
 public:
-  TTEntry();
-  TTEntry(Key k, Value v, ValueType t, Depth d, Move m, int generation);
-  Key key() const { return key_; }
-  Depth depth() const { return Depth(depth_); }
+  void save(uint32_t k, Value v, ValueType t, Depth d, Move m, int g, Value statV, Value kd) {
+
+      key32 = k;
+      data = (m & 0x1FFFF) | (t << 21) | (g << 23);
+      value16     = int16_t(v);
+      depth16     = int16_t(d);
+      staticValue = int16_t(statV);
+      staticValueMargin  = int16_t(kd);
+  }
+
+  uint32_t key() const { return key32; }
+  Depth depth() const { return Depth(depth16); }
   Move move() const { return Move(data & 0x1FFFF); }
-  Value value() const { return Value(value_); }
-  ValueType type() const { return ValueType((data >> 20) & 7); }
-  int generation() const { return (data >> 23); }
+  Value value() const { return Value(value16); }
+  ValueType type() const { return ValueType((data >> 21) & 3); }
+  int generation() const { return data >> 23; }
+  Value static_value() const { return Value(staticValue); }
+  Value static_value_margin() const { return Value(staticValueMargin); }
 
 private:
-  Key key_;
+  uint32_t key32;
   uint32_t data;
-  int16_t value_;
-  int16_t depth_;
+  int16_t value16;
+  int16_t depth16;
+  int16_t staticValue;
+  int16_t staticValueMargin;
 };
 
-/// The transposition table class.  This is basically just a huge array
-/// containing TTEntry objects, and a few methods for writing new entries
+
+/// This is the number of TTEntry slots for each cluster
+const int ClusterSize = 4;
+
+/// TTCluster consists of ClusterSize number of TTEntries.
+/// Size of TTCluster must not be bigger than a cache line size.
+/// In case it is less, it should be padded to guarantee always aligned accesses.
+
+struct TTCluster {
+  TTEntry data[ClusterSize];
+};
+
+
+/// The transposition table class. This is basically just a huge array
+/// containing TTCluster objects, and a few methods for writing new entries
 /// and reading new ones.
 
 class TranspositionTable {
 
+  TranspositionTable(const TranspositionTable&);
+  TranspositionTable& operator=(const TranspositionTable&);
+
 public:
-  TranspositionTable(unsigned mbSize);
+  TranspositionTable();
   ~TranspositionTable();
-  void set_size(unsigned mbSize);
+  void set_size(size_t mbSize);
   void clear();
-  void store(const Position &pos, Value v, Depth d, Move m, ValueType type);
-  TTEntry* retrieve(const Position &pos) const;
+  void store(const Key posKey, Value v, ValueType type, Depth d, Move m, Value statV, Value kingD);
+  TTEntry* retrieve(const Key posKey) const;
   void new_search();
-  void insert_pv(const Position &pos, Move pv[]);
-  int full();
+  TTEntry* first_entry(const Key posKey) const;
 
 private:
-  inline TTEntry* first_entry(const Position &pos) const;
-
-  unsigned size;
-  int writes;
-  TTEntry* entries;
+  size_t size;
+  TTCluster* entries;
   uint8_t generation;
 };
 
+extern TranspositionTable TT;
 
-////
-//// Constants and variables
-////
 
-// Default transposition table size, in megabytes:
-const int TTDefaultSize = 32;
+/// TranspositionTable::first_entry returns a pointer to the first
+/// entry of a cluster given a position. The lowest order bits of the key
+/// are used to get the index of the cluster.
+
+inline TTEntry* TranspositionTable::first_entry(const Key posKey) const {
 
+  return entries[uint32_t(posKey) & (size - 1)].data;
+}
 
 #endif // !defined(TT_H_INCLUDED)
@@ -26,7 +26,7 @@
 /// A TTEntry needs 128 bits to be stored
 ///
-/// bit    0-63: key
-/// bit   64-95: data
-/// bit  96-111: value
-/// bit 112-127: depth
+/// bit  0-31: key
+/// bit 32-63: data
+/// bit 64-79: value
+/// bit 80-95: depth
+/// bit 96-111: static value
+/// bit 112-127: margin of static value
 ///
 /// the 32 bits of the data field are so defined
@@ -26,7 +26,7 @@
 ///
 /// bit  0-16: move
-/// bit 17-19: not used
-/// bit 20-22: value type
+/// bit 17-20: not used
+/// bit 21-22: value type
 /// bit 23-31: generation
 